<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Objects;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;

/**
 * ImageItem
 *
 * @property int height
 * @property array links
 * @property string uuid
 * @property string type
 * @property int width
 * @property string uri
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\NewsML\Objects
 * @since   Infomaker\Everyware\NewsML\Objects\ImageItem 1.0.0
 */
class ImageItem extends Item
{

    public function getCropForRatio(string $ratio)
    {
        return Arr::get($this->getMultiValue('crop'), $ratio, []);
    }

    public function hasCropForRatio(string $ratio): bool
    {
        return !empty($this->getCropForRatio($ratio));
    }

    public function hasOriginalSizes(): bool
    {
        if (!isset($this->width, $this->height)) {
            return false;
        }

        return $this->width < 0 && $this->height < 0;
    }

    public function fileType(string $defaultType = 'jpg'): string
    {
        $fileType = pathinfo($this->uri, PATHINFO_EXTENSION);
        return strtolower(empty($fileType) ? $defaultType : $fileType);
    }
}
