<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use SimpleXMLIterator;

/**
 * DataParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\NewsML\Parsers
 * @since   Infomaker\Everyware\NewsML\Parsers\DataParser 1.0.0
 */
class DataParser implements ItemParser
{

    /**
     * Contains a group of parsers
     *
     * @var ItemParser[]
     */
    private $parsers;

    /**
     * ObjectParser constructor.
     *
     * @since 0.1
     *
     * @param ItemParser[]
     */
    public function __construct(array $parsers = [])
    {
        $this->parsers = $parsers;
    }

    public function parse(SimpleXMLIterator $element): array
    {
        if (property_exists($element, 'data')) {
            $dataIterator = $element->data->children();

            $data = [];
            foreach ($dataIterator as $tag => $item) {
                $parser = Arr::get($this->parsers, $tag, null);
                $data[$tag] = $parser instanceof ItemParser ? $parser->parse($item) : (string)$item;
            }

            return $data;
        }

        return [];
    }
}
