<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use SimpleXMLIterator;

/**
 * ImageLinkParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\NewsML\Parsers
 * @since   Infomaker\Everyware\NewsML\Parsers\ImageLinkParser 1.0.0
 */
class ImageLinkParser extends LinkParser implements ItemParser
{

    public const LINK_TYPE = 'x-im/image';

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new ImageItem($element);
        $item->fill(array_replace($this->getImageData($item), $this->getImageLinks($element)));

        return [$item];
    }

    public function getImageLinks(SimpleXMLIterator $element): array
    {
        $links = [];
        foreach ($this->parseLinks($element) as $link) {
            if ($link instanceof Item) {
                $attributes = $link->getItemAttributes();
                $linkRel = Arr::get($attributes, 'rel', 'link');
                $title = Arr::get($attributes, 'title', '');

                if ($linkRel === 'crop' && $title !== '') {
                    $links[$linkRel][$title] = $this->extractCropValues($attributes);
                } else {
                    $links[$linkRel][] = $link->getItemAttributes();
                }
            }
        }

        return $links;
    }

    protected function extractCropValues(array $crop): array
    {
        $values = array_map([
            $this,
            'formatFloatCropNumber'
        ], explode('/', str_replace('im://crop/', '', $crop['uri'])));

        return [
            'x' => $values[0],
            'y' => $values[1],
            'width' => $values[2],
            'height' => $values[3]
        ];
    }

    protected function getImageData(Item $item): array
    {
        return array_replace([
            'type' => $item->getType(),
            'uuid' => $item->getItemAttribute('uuid'),
            'width' => 0,
            'height' => 0
        ], (array)($item->getChild('data') ?? []));
    }

    /**
     * Converts 1 and 0 since Imengine shows strange behaviour when passed those
     *
     * @param $number
     *
     * @return float
     */
    protected function formatFloatCropNumber(float $number): float
    {
        $number = (float)sprintf('%.5f', $number + 0);
        if (\strlen((string)$number) === 1) {
            if ($number === 1) {
                $number = 0.99999;
            } elseif ($number === 0) {
                $number = 0.00001;
            }
        }

        return $number;
    }
}
