<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\TypeParser;
use SimpleXMLIterator;

class PdfParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/pdf';

    /**
     * @var LinkParser
     */
    protected $linkParser;

    public function __construct()
    {
        $this->linkParser = new LinkParser();
    }

    public function parse(SimpleXMLIterator $element): array
    {

        $item = new Item($element);
        $item->fill($this->getLinkAttributes($element));

        return [$item];
    }

    private function getLinkAttributes(SimpleXMLIterator $element): array
    {
        $links = $this->linkParser->parseLinks($element);
        if (!empty($links)) {
            foreach ($links as $index => $link) {
                if ($link instanceof Item) {
                    $attributes = $link->getItemAttributes();
                    if (Arr::get($attributes, 'rel', '') === 'self') {
                        return [
                            'title' => $this->getTitle($link),
                            'file' => $this->getFileName($link),
                            'url' => $this->createUrl($link),
                        ];
                    }
                }
            }
        }

        return [];
    }

    private function getFileName(Item $item): string
    {
        return str_replace('im://pdf/', '', (string)$item->getItemAttribute('uri', ''));
    }

    private function getTitle(Item $item): string
    {
        $data = $item->getChild('data');
        return isset($data->text) ? (string)$data->text : '';
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }

    private function createUrl(Item $item): string
    {
        if (!defined('CF_PDF')) {
            error_log('CF_PDF is not defined and can therefor not serve PDFs');

            return '';
        }

        return CF_PDF . '/' . $this->getFileName($item);
    }
}
